/*
 * gtkpasman
 * Copyright (C) 2011 David Guglielmi
 *
 * gtk_ui.c
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <stdlib.h>
#include <gtk_ui/gtk_ui.h>

#ifdef HAVE_CONFIG_H
  #include <config.h>
#endif

/* FIXME: No include this */
#include "tree.h"
#include "passwd.h"
#include "gtkpasman_config.h"
#include "configread.h"

/* Local used function */
GtkWidget* create_root_window (void);
GtkWidget* pasman_askpassword_widget_new (void);

static void window_state_change (GtkWidget *, GdkEventWindowState *, gpointer);
static void select_file (void);
static void show_about_dialog (void);
static void pasman_window_swap_state (GtkWindow *);
static void create_gtkpasman_config (void);
static void change_passwd_entry_visibility (GtkToggleButton*, gpointer);
static void change_pass (GtkWidget *passtxt);
static void change_config (GtkWidget *dialog);
static void show_askpassword (void);

/* FIXME: do not use global variable */
/* widgets to be used in several places */

/* Setup dialog entry widget */
GtkWidget *passwd_file_entry;
GtkWidget *ssh_cmd_entry;
GtkWidget *ftp_cmd_entry;
GtkWidget *telnet_cmd_entry;
GtkWidget *http_cmd_entry;
GtkWidget *cssh_cmd_entry;

void
init_gtk_ui (void)
{

  int argc = 1;
  const char* args[] = {"", "\0"};
  char** argv = (char**)&args;

  gtk_init (&argc, &argv);

}

void
start_gtk_ui (int initial_password)
{

  GtkWidget *window;
  GtkWidget *prompt_passwd;
 
  window = create_root_window ();

  /* Don't display passphrase dialog */    
  if (initial_password == 0)
   {
    prompt_passwd = g_object_get_data (G_OBJECT (window),\
                      "pasman_prompt_passwd");
    gtk_widget_hide (prompt_passwd);
   }

  gtk_main ();
}

/*
 * create_root_window
 *      create the list window and all the widgets
 */

GtkWidget*
create_root_window ()
{

  GtkWidget *window;
  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *button;
  GtkWidget *menu;
  GtkWidget *menu_bar;
  GtkWidget *menu_item;
  GtkWidget *tree_window;
  GtkWidget *tree_view;
  GtkWidget *label;
  GtkWidget *entry;
  GtkWidget *toolbar;
  GtkWidget *clipboard;
  GtkWidget *prompt_passwd;
  GtkWidget *statusbar;
  GtkToolItem *tool_item;
  GtkStatusIcon *tray_icon;
  GtkAccelGroup *accel_group;
  GdkPixbuf *pixbuf;

  /* main window */
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window), "GtkPasman");
  gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);

  gtk_window_set_default_size (GTK_WINDOW (window), -1, 350);

#if GTK_CHECK_VERSION(3,0,0)
 pixbuf = gtk_widget_render_icon_pixbuf (window,\
                                        GTK_STOCK_DIALOG_AUTHENTICATION,\
                                        GTK_ICON_SIZE_BUTTON);
#else
  pixbuf = gtk_widget_render_icon (window,\
                                  GTK_STOCK_DIALOG_AUTHENTICATION,\
                                  GTK_ICON_SIZE_BUTTON,\
                                  NULL);
#endif

  gtk_window_set_default_icon (pixbuf);
  g_object_unref (pixbuf);

  accel_group = gtk_accel_group_new ();
  gtk_window_add_accel_group (GTK_WINDOW(window), accel_group);

  g_signal_connect_swapped (G_OBJECT (window), "destroy",
		      G_CALLBACK (gtk_main_quit), NULL);

  g_signal_connect (G_OBJECT (window), "window-state-event",\
    G_CALLBACK (window_state_change), NULL);

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (window), vbox);

  tree_view = pasman_tree_view_new ();

  /* menus */
  menu_bar = gtk_menu_bar_new ();

    menu = gtk_menu_new ();

    menu_item = gtk_menu_item_new_with_mnemonic ("_Open File");
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
    g_signal_connect (G_OBJECT (menu_item), "activate",\
      G_CALLBACK (select_file), NULL);

    menu_item = gtk_image_menu_item_new_from_stock (\
      GTK_STOCK_PREFERENCES, NULL);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
    g_signal_connect (G_OBJECT (menu_item), "activate",\
      G_CALLBACK (create_gtkpasman_config), NULL);

    menu_item = gtk_separator_menu_item_new ();
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);

    menu_item = gtk_image_menu_item_new_from_stock (GTK_STOCK_QUIT, NULL);
    g_signal_connect (G_OBJECT (menu_item), "activate",\
      G_CALLBACK (gtk_main_quit), NULL);
#if GTK_CHECK_VERSION(3,0,0)
    gtk_widget_add_accelerator (menu_item, "activate", accel_group,\
      GDK_KEY_q, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
#else
    gtk_widget_add_accelerator (menu_item, "activate", accel_group,\
      GDK_q, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
#endif
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);

  menu_item = gtk_menu_item_new_with_mnemonic ("_File");
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), menu);
  gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), menu_item);

    menu = gtk_menu_new ();

    menu_item = gtk_menu_item_new_with_label ("Copy command line");
    g_signal_connect_swapped (G_OBJECT (menu_item), "activate",\
		  G_CALLBACK (button_copy_cmd_line), tree_view);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);

    menu_item = gtk_menu_item_new_with_label ("Copy password");
    g_signal_connect_swapped (G_OBJECT (menu_item), "activate",\
		  G_CALLBACK (button_copy_passwd), tree_view);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);

    menu_item = gtk_menu_item_new_with_label ("Copy password & Execute");
    g_signal_connect_swapped (G_OBJECT (menu_item), "activate",\
		  G_CALLBACK (button_copy_passwd_and_execute), tree_view);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);

    menu_item = gtk_image_menu_item_new_from_stock (GTK_STOCK_EXECUTE, NULL);
    g_signal_connect_swapped (G_OBJECT (menu_item), "activate",\
		  G_CALLBACK (button_execute), tree_view);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);

    menu_item = gtk_separator_menu_item_new ();
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);

    menu_item = gtk_image_menu_item_new_from_stock (GTK_STOCK_GO_FORWARD, NULL);
    g_signal_connect (G_OBJECT (menu_item), "activate",\
      G_CALLBACK (find_next_in_tree_view), tree_view);
#if GTK_CHECK_VERSION(3,0,0)
    gtk_widget_add_accelerator (menu_item, "activate", accel_group,\
      GDK_KEY_n, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
#else
    gtk_widget_add_accelerator (menu_item, "activate", accel_group,\
      GDK_n, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
#endif
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);

    menu_item = gtk_image_menu_item_new_from_stock (GTK_STOCK_GO_BACK, NULL);
    g_signal_connect (G_OBJECT (menu_item), "activate",\
      G_CALLBACK (find_prev_in_tree_view), tree_view);
#if GTK_CHECK_VERSION(3,0,0)
    gtk_widget_add_accelerator (menu_item, "activate", accel_group,\
      GDK_KEY_p, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
#else
    gtk_widget_add_accelerator (menu_item, "activate", accel_group,\
      GDK_p, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
#endif
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);

  menu_item = gtk_menu_item_new_with_mnemonic ("_Action");
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), menu);
  gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), menu_item);

    menu = gtk_menu_new ();

    menu_item = gtk_menu_item_new_with_label ("Collapse all");
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
    g_signal_connect_swapped (G_OBJECT (menu_item), "activate",\
      G_CALLBACK (gtk_tree_view_collapse_all), tree_view);

    menu_item = gtk_menu_item_new_with_label ("Expand a little");
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
    g_signal_connect_swapped (G_OBJECT (menu_item), "activate",\
      G_CALLBACK (tree_expand_little), tree_view);

    menu_item = gtk_menu_item_new_with_label ("Expand all");
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
    g_signal_connect_swapped (G_OBJECT (menu_item), "activate",\
      G_CALLBACK (gtk_tree_view_expand_all), tree_view);

    menu_item = gtk_menu_item_new_with_label ("Unselect all");
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
    g_signal_connect_swapped (G_OBJECT (menu_item), "activate",\
      G_CALLBACK (tree_unselectall), tree_view);

    menu_item = gtk_separator_menu_item_new ();
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);

    menu_item = gtk_image_menu_item_new_from_stock (GTK_STOCK_REFRESH, NULL);
    g_signal_connect (G_OBJECT (menu_item), "activate",\
      G_CALLBACK (show_askpassword), NULL);
#if GTK_CHECK_VERSION(3,0,0)
    gtk_widget_add_accelerator (menu_item, "activate", accel_group,\
      GDK_KEY_r, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
#else
    gtk_widget_add_accelerator (menu_item, "activate", accel_group,\
      GDK_r, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
#endif
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);

  menu_item = gtk_menu_item_new_with_mnemonic ("_View");
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), menu);
  gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), menu_item);

    menu = gtk_menu_new ();
    menu_item = gtk_image_menu_item_new_from_stock (GTK_STOCK_ABOUT, NULL);
    g_signal_connect (G_OBJECT (menu_item), "activate",\
      G_CALLBACK (show_about_dialog), NULL);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);

  menu_item = gtk_menu_item_new_with_mnemonic ("_Help");
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), menu);
  gtk_menu_shell_append (GTK_MENU_SHELL (menu_bar), menu_item);

  gtk_box_pack_start (GTK_BOX (vbox), menu_bar, FALSE, FALSE, 0);

  prompt_passwd = pasman_askpassword_widget_new ();

  gtk_box_pack_start (GTK_BOX (vbox), prompt_passwd,\
    FALSE, FALSE, 0);

  toolbar = gtk_toolbar_new ();

  gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), GTK_TOOLBAR_BOTH);
  gtk_toolbar_set_show_arrow (GTK_TOOLBAR (toolbar), FALSE);

  tool_item = gtk_tool_button_new_from_stock (GTK_STOCK_COPY);
  gtk_tool_button_set_label (GTK_TOOL_BUTTON (tool_item), "Copy cmd line");
  gtk_tool_item_set_tooltip_text (tool_item, "Copy command line.");
  g_signal_connect_swapped (G_OBJECT (tool_item), "clicked",\
    G_CALLBACK (button_copy_cmd_line), tree_view);
  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), tool_item, 0);

  tool_item = gtk_tool_button_new_from_stock (GTK_STOCK_COPY);
  gtk_tool_button_set_label  (GTK_TOOL_BUTTON (tool_item), "Copy passwd");
  gtk_tool_item_set_tooltip_text (tool_item, "Copy password.");
  g_signal_connect_swapped (G_OBJECT (tool_item), "clicked",\
    G_CALLBACK (button_copy_passwd), tree_view);
  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), tool_item, -1);

  tool_item = gtk_tool_button_new_from_stock (GTK_STOCK_COPY);
  gtk_tool_button_set_label  (GTK_TOOL_BUTTON (tool_item),\
    "Copy & Execute");
  gtk_tool_item_set_tooltip_text (tool_item,\
    "Copy password and execute command line.");
  g_signal_connect_swapped (G_OBJECT (tool_item), "clicked",\
    G_CALLBACK (button_copy_passwd_and_execute), tree_view);
  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), tool_item, -1);

  tool_item = gtk_tool_button_new_from_stock (GTK_STOCK_EXECUTE);
  gtk_tool_item_set_tooltip_text (tool_item, "Execute command line.");
  g_signal_connect_swapped (G_OBJECT (tool_item), "clicked",\
    G_CALLBACK (button_execute), tree_view);
  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), tool_item, -1);

  gtk_box_pack_start (GTK_BOX (vbox), toolbar, FALSE, FALSE, 0);

  /* scrollwindow */
  tree_window = gtk_scrolled_window_new (NULL, NULL);

  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (tree_window),\
    GTK_POLICY_NEVER,\
    GTK_POLICY_AUTOMATIC);

  gtk_container_add (GTK_CONTAINER (tree_window), tree_view);
  gtk_box_pack_start (GTK_BOX (vbox), tree_window, TRUE, TRUE, 0);

  /* search entry */
  hbox = gtk_hbox_new (FALSE, 2);
  gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);

  label = gtk_label_new ("Search : ");
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);

  entry = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
  g_signal_connect_swapped (G_OBJECT (entry), "changed",\
			     G_CALLBACK (search_in_list), entry);
  g_signal_connect_swapped (G_OBJECT (entry), "activate",\
			     G_CALLBACK (search_in_list), entry);

  button = gtk_button_new ();
  gtk_button_set_image (GTK_BUTTON (button),\
    gtk_image_new_from_stock (GTK_STOCK_GO_FORWARD, GTK_ICON_SIZE_BUTTON));
  g_signal_connect (G_OBJECT (button), "clicked",\
		      G_CALLBACK (find_next_in_tree_view), tree_view); 
  gtk_widget_set_tooltip_text (button, "Find forward.");
  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);

  button = gtk_button_new ();
  gtk_button_set_image (GTK_BUTTON (button),\
    gtk_image_new_from_stock (GTK_STOCK_GO_BACK, GTK_ICON_SIZE_BUTTON));
  gtk_widget_set_tooltip_text (button, "Find back.");
  g_signal_connect (G_OBJECT (button), "clicked",\
		      G_CALLBACK (find_prev_in_tree_view), tree_view); 
  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);

  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

  statusbar = gtk_statusbar_new ();

  gtk_box_pack_start (GTK_BOX (vbox), statusbar, FALSE, FALSE, 0);

  /* show all */
  gtk_widget_show_all (window);

  /* then add hidden entry used to have paste selections */
  clipboard = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (vbox), clipboard, FALSE, TRUE, 0);
  gtk_widget_show (clipboard);
  gtk_widget_hide (clipboard);
  
  /* Gtk tray icon: the bcm request */
  tray_icon = gtk_status_icon_new ();

  gtk_status_icon_set_from_icon_name (tray_icon,\
    GTK_STOCK_DIALOG_AUTHENTICATION);
  gtk_status_icon_set_tooltip_text (tray_icon, "GtkPasman");
  gtk_status_icon_set_visible (tray_icon, TRUE);

  g_signal_connect_swapped (G_OBJECT(tray_icon), "activate", 
    G_CALLBACK (pasman_window_swap_state), window);

/*
  Set Global Widget
*/
  g_object_set_data (G_OBJECT (window), "pasman_search_entry", entry);
  g_object_set_data (G_OBJECT (window), "pasman_clipboard", clipboard);
  g_object_set_data (G_OBJECT (window), "pasman_prompt_passwd", prompt_passwd);
  g_object_set_data (G_OBJECT (window), "pasman_tree_view", tree_view);

  return window;

}

/*
 * About dialog.
 *
 */
static void
show_about_dialog (void)
{
  GtkWidget *dialog;
  GdkPixbuf *pixbuf;

  const gchar *authors [] = { "Cyril Bellot <jcpp@users.sourceforge.net>",\
                        "David Guglielmi <dg@accelance.fr>",\
                        NULL};

  dialog = gtk_about_dialog_new ();

#if GTK_CHECK_VERSION(3,0,0)
  pixbuf = gtk_widget_render_icon_pixbuf (dialog,\
                                        GTK_STOCK_DIALOG_AUTHENTICATION,\
                                        GTK_ICON_SIZE_DIALOG);
#else
  pixbuf = gtk_widget_render_icon (dialog,\
                                  GTK_STOCK_DIALOG_AUTHENTICATION,\
                                  GTK_ICON_SIZE_DIALOG,\
                                  NULL);
#endif

  gtk_about_dialog_set_logo (GTK_ABOUT_DIALOG (dialog), pixbuf);

  g_object_unref (pixbuf);

  gtk_about_dialog_set_logo (GTK_ABOUT_DIALOG (dialog), pixbuf);

  gtk_about_dialog_set_authors (GTK_ABOUT_DIALOG (dialog), authors);

#if (defined(PACKAGE_NAME) && defined(PACKAGE_VERSION))
  gtk_about_dialog_set_program_name (GTK_ABOUT_DIALOG (dialog), PACKAGE_NAME);
  gtk_about_dialog_set_version (GTK_ABOUT_DIALOG (dialog), PACKAGE_VERSION);
#else
  #error Unable to find PACKAGE_NAME or/and PACKAGE_VERSION, please define it !
#endif

  gtk_about_dialog_set_website (GTK_ABOUT_DIALOG (dialog),\
    "http://gtkpasman.sourceforge.net/");

  gtk_about_dialog_set_comments (GTK_ABOUT_DIALOG (dialog),\
    ("lightweight password manager based on GPG and GTK+."));

  gtk_about_dialog_set_copyright (GTK_ABOUT_DIALOG (dialog),\
    ("Copyright \xc2\xa9 2011 OpenWide Outsourcing (ex .Accelance MSP)"));

  gtk_window_set_transient_for (GTK_WINDOW (dialog),\
    GTK_WINDOW (gtk_window_list_toplevels ()->data));

  gtk_dialog_run (GTK_DIALOG (dialog));
  gtk_widget_destroy (dialog);

}

/*
 * create_gtkpasman_config
 *      create the configuration dialog
 */
void
create_gtkpasman_config ()
{
  GtkWidget *dialog;
  GtkWidget *frame;
  GtkWidget *label;
  GtkWidget *hbox;
  GtkWidget *vbox;
  GtkWidget *button;
  GtkWidget *content_area;

  /* main window (modal) */
  dialog = gtk_dialog_new_with_buttons ("Setup",\
    GTK_WINDOW (gtk_window_list_toplevels ()->data),\
    GTK_DIALOG_MODAL,\
    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,\
    GTK_STOCK_OK, GTK_RESPONSE_OK,\
    NULL);
  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);

  content_area  = gtk_dialog_get_content_area (GTK_DIALOG (dialog));

  /* password file */
  frame = gtk_frame_new ("Password file");
  gtk_box_pack_start (GTK_BOX (content_area),
		      frame, FALSE, FALSE, 0);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (frame), hbox);

  label = gtk_label_new ("Default file");
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);

  passwd_file_entry = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (passwd_file_entry), passwdfile);
  gtk_box_pack_start (GTK_BOX (hbox), passwd_file_entry, TRUE, TRUE, 0);

  button = gtk_button_new ();
  gtk_button_set_image (GTK_BUTTON (button),\
    gtk_image_new_from_stock (GTK_STOCK_OPEN, GTK_ICON_SIZE_BUTTON));
  g_signal_connect_swapped (G_OBJECT (button), "clicked",\
    G_CALLBACK (select_file), NULL);

  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 2);

  /* Command lines */
  frame = gtk_frame_new ("Commands");
  gtk_box_pack_start (GTK_BOX (content_area),
		      frame, FALSE, FALSE, 2);

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (frame), vbox);

  label = gtk_label_new ("SSH command:");
  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);

    ssh_cmd_entry = gtk_entry_new ();
    gtk_entry_set_text (GTK_ENTRY (ssh_cmd_entry), sshcmd);
    gtk_box_pack_start (GTK_BOX (vbox), ssh_cmd_entry, FALSE, FALSE, 0);

  label = gtk_label_new ("CSSH command:");
  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);

    cssh_cmd_entry = gtk_entry_new ();
    gtk_entry_set_text (GTK_ENTRY (cssh_cmd_entry), csshcmd);
    gtk_box_pack_start (GTK_BOX (vbox), cssh_cmd_entry, FALSE, FALSE, 0);

  label = gtk_label_new ("FTP command:");
  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);

    ftp_cmd_entry = gtk_entry_new ();
    gtk_entry_set_text (GTK_ENTRY (ftp_cmd_entry), ftpcmd);
    gtk_box_pack_start (GTK_BOX (vbox), ftp_cmd_entry, FALSE, FALSE, 0);

  label = gtk_label_new ("TELNET command:");
  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);

    telnet_cmd_entry = gtk_entry_new ();
    gtk_entry_set_text (GTK_ENTRY (telnet_cmd_entry), telnetcmd);
    gtk_box_pack_start (GTK_BOX (vbox), telnet_cmd_entry, FALSE, FALSE, 0);

  label = gtk_label_new ("HTTP command:");
  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);

    http_cmd_entry = gtk_entry_new ();
    gtk_entry_set_text (GTK_ENTRY (http_cmd_entry), httpcmd);
    gtk_box_pack_start (GTK_BOX (vbox), http_cmd_entry, FALSE, FALSE, 0);

  gtk_widget_show_all (dialog);

  switch (gtk_dialog_run (GTK_DIALOG (dialog)))
   {

     case GTK_RESPONSE_OK:
       change_config (dialog);
       break;
       ;;
     
     default:
       gtk_widget_destroy (dialog);
   }

}

/*
 * select_file 
 *      select a password file
 */
static void
select_file (void)
{

  GtkWidget *file_chooser;

  file_chooser =  gtk_file_chooser_dialog_new  ("Select a new password file",\
    GTK_WINDOW (gtk_window_list_toplevels ()->data),
    GTK_FILE_CHOOSER_ACTION_OPEN,
    GTK_STOCK_CANCEL,
    GTK_RESPONSE_CANCEL,
    GTK_STOCK_OPEN,
    GTK_RESPONSE_OK,
    NULL);

 switch (gtk_dialog_run (GTK_DIALOG (file_chooser)))
 {
 
  case GTK_RESPONSE_OK:

    g_free (passwdfile);
    passwdfile = g_strdup (\
      gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_chooser)));

    config_write ();
    show_askpassword ();
    ;;
  default:
    break;
  }

  gtk_widget_destroy (file_chooser);

}

GtkWidget*
pasman_askpassword_widget_new (void)
{

  GtkWidget *entry;
  GtkWidget *button;
  GtkWidget *hbox;
  GtkWidget *vbox;

  vbox = gtk_vbox_new (FALSE, 2);
  gtk_box_pack_start (GTK_BOX (vbox),\
    gtk_label_new ("Please enter your PGP Passphrase:") , FALSE, FALSE, 2);

  gtk_container_set_border_width (GTK_CONTAINER (vbox), 2);

  hbox = gtk_hbox_new (FALSE, 2);

  /* password entry */
  entry = gtk_entry_new ();
  gtk_entry_set_visibility (GTK_ENTRY (entry), FALSE);

  gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);

  g_signal_connect (entry, "activate",\
    G_CALLBACK (change_pass), NULL);
  
  g_signal_connect_swapped (entry, "activate",\
    G_CALLBACK (gtk_widget_hide), vbox);

  button = gtk_toggle_button_new ();
  
  gtk_button_set_image (GTK_BUTTON (button),\
    gtk_image_new_from_stock (GTK_STOCK_FIND, GTK_ICON_SIZE_MENU));

  g_signal_connect (button, "toggled",\
    G_CALLBACK (change_passwd_entry_visibility), (gpointer)entry);

  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);

  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

  hbox = gtk_hbox_new (FALSE, 2);

  gtk_box_pack_start (GTK_BOX (hbox),\
    gtk_alignment_new (0.0, 0.0, 0.0, 0.0), TRUE, TRUE, 0);

  button = gtk_button_new_from_stock (GTK_STOCK_OK);
  g_signal_connect_swapped (button, "clicked",\
    G_CALLBACK (change_pass), entry);
  g_signal_connect_swapped (button, "clicked",\
    G_CALLBACK (gtk_widget_hide), vbox);

  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);

  button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
  g_signal_connect_swapped (button, "clicked",\
    G_CALLBACK (gtk_widget_hide), vbox);

  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);

  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

  g_signal_connect_swapped (G_OBJECT (vbox), "show",\
    G_CALLBACK (gtk_widget_grab_focus), entry);  

  return vbox;

}

/*
 * show_askpassword
 *      create the asking passphrase window
 */
void
show_askpassword (void)
{

  GtkWidget *window;
  GtkWidget *prompt_passwd;
  GtkWidget *tree_view;
  GtkTreeModel *tree_model;
  
  window = gtk_window_list_toplevels ()->data;
  prompt_passwd = g_object_get_data (G_OBJECT (window), "pasman_prompt_passwd");
  
  gtk_widget_show (prompt_passwd);
  
  tree_view = g_object_get_data (G_OBJECT (window), "pasman_tree_view");
  tree_model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree_view));
  
  emptytree (GTK_TREE_STORE (tree_model));
}

/*
 * Change password visibility state.
 *
 */

static void
change_passwd_entry_visibility (GtkToggleButton *togglebutton,\
  gpointer user_data)
{

  gtk_entry_set_visibility (GTK_ENTRY (user_data),\
    gtk_toggle_button_get_active (togglebutton));

}

/*
 *  Notification Area widget.
 *
 */

void
window_state_change (GtkWidget *widget, GdkEventWindowState *event,\
  gpointer user_data)
{

  gtk_window_set_skip_taskbar_hint (GTK_WINDOW (widget),\
    (event->new_window_state == GDK_WINDOW_STATE_ICONIFIED));

} 

void
pasman_window_swap_state (GtkWindow *window)
{

 GdkWindow *w;

 w = gtk_widget_get_window (GTK_WIDGET (window));

 if (gdk_window_get_state(w) == GDK_WINDOW_STATE_ICONIFIED)
    gtk_window_deiconify (window);
 else
    gtk_window_iconify (window);

}

/*
 *  Old function
 *
 */
 
/* ###########################################################################*/

/*
 * change_pass
 *      change the password and try to fill the tree
 */
void
change_pass (GtkWidget *passtxt)
{

  GtkWidget *window;
  GtkWidget *entry;
  GtkWidget *tree_view;
  GtkTreeModel *tree_model;

  window = gtk_window_list_toplevels ()->data;

  /* read password from the entry widget */
  globalpasswd = g_strdup (gtk_entry_get_text (GTK_ENTRY (passtxt)));

  gtk_entry_set_text (GTK_ENTRY (passtxt), "");

  tree_view = g_object_get_data (G_OBJECT (window), "pasman_tree_view");
  tree_model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree_view));

  /* empty tree */
  emptytree (GTK_TREE_STORE (tree_model));

  /* read file and fill tree */
  decrypted_passwds = filltree (GTK_TREE_STORE (tree_model));

  entry = g_object_get_data (G_OBJECT (window), "pasman_search_entry");
  
  gtk_widget_grab_focus (entry);

  /* No need to keep passphrase in memory now... */
  g_free (globalpasswd);
}

/*
 * select_command
 *      put command in the clipboard
 */
void
select_command (gchar **data)
{
  gchar *txt;
  GtkWidget *window;
  GtkWidget *clipboard;

  window = gtk_window_list_toplevels ()->data;
  clipboard = g_object_get_data (G_OBJECT (window), "pasman_clipboard");

  if (data)
   {
    txt = data[1];
    gtk_entry_set_text (GTK_ENTRY (clipboard), txt);
    gtk_editable_select_region (GTK_EDITABLE (clipboard), 0, -1);
    gtk_editable_copy_clipboard (GTK_EDITABLE (clipboard));
  }
}

/*
 * select_passwd
 *      put password in the clipboard
 */
void
select_passwd (gchar **data)
{
  gchar *txt;

  GtkWidget *window;
  GtkWidget *clipboard;

  window = gtk_window_list_toplevels ()->data;
  clipboard = g_object_get_data (G_OBJECT (window), "pasman_clipboard");

  if (data != NULL) {
    txt = data[0];
    gtk_entry_set_text (GTK_ENTRY (clipboard), txt);
    gtk_editable_select_region (GTK_EDITABLE (clipboard), 0, -1);
    gtk_editable_copy_clipboard (GTK_EDITABLE (clipboard));
  }
}

/*
 * select_passwd_and_exec_command
 *      put password in the clipboard
 *      execute command
 */
void
select_passwd_and_exec_command (gchar **data)
{
  gchar *txt;
  int ret;

  select_passwd (data);
  if (data != NULL) {
    txt = data[1];
    ret = system (txt);
  }

  (void)ret;

}

/*
 * exec_command
 *      execute command
 */
void
exec_command (gchar **data)
{
  gchar *txt;
  int ret;

  if (data != NULL) {
    txt = data[1];
    ret = system (txt);
  }
  
  (void)ret;
  
}

/*
 * exec_multiple
 *      execute command whith multiple selection
 */
void
exec_multiple (gchar *hostlist)
{
  gchar c = ' ', *s;
  gchar *cmd, *res, *tmp;
  int ret;

  cmd = g_strdup (csshcmd);
  
  /* replace %u by values */
  res = g_strdup ("");
  tmp = g_strdup ("");
  for (s = cmd; *s != '\0'; ++s) {
    if (*s == '%' && *(s + 1) != '\0') {
	  if ((c = *(s + 1)) == 'h') {
        tmp = g_strconcat (res, hostlist, NULL);
      }
	  else {
	    tmp = g_strdup_printf ("%s%c", res, *s);
	  }
      s++; 
    }
    else { 
      tmp = g_strdup_printf ("%s%c", res, *s);
    }
    g_free (res);
    res = g_strdup (tmp);
    g_free (tmp);
  } 

  /* execute command */
  ret = system(res);

  /* free resources */
  g_free (cmd);
  g_free (res);
  
  (void)ret;
}

/*
 * menu_copy_passwd
 *      callback for menu
 */ 
void
menu_copy_passwd (gpointer data)
{   
  if (decrypted_passwds)
    select_passwd (data);
}
    
/*
 * menu_copy_passwd_and_execute
 *      callback for menu
 */ 
void
menu_copy_passwd_and_execute (gpointer data)
{
  if (decrypted_passwds)
    select_passwd_and_exec_command (data);
}



/*
 * change_config
 *      store the values from the configuration window
 */
void
change_config (GtkWidget *dialog)
{

  g_free (passwdfile);
  g_free (sshcmd);
  g_free (ftpcmd);
  g_free (telnetcmd);
  g_free (httpcmd);
  g_free (csshcmd);

  passwdfile = g_strdup (gtk_entry_get_text (GTK_ENTRY (passwd_file_entry)));
  sshcmd = g_strdup (gtk_entry_get_text (GTK_ENTRY (ssh_cmd_entry)));
  ftpcmd = g_strdup (gtk_entry_get_text (GTK_ENTRY (ftp_cmd_entry)));
  telnetcmd = g_strdup (gtk_entry_get_text (GTK_ENTRY (telnet_cmd_entry)));
  httpcmd = g_strdup (gtk_entry_get_text (GTK_ENTRY (http_cmd_entry)));
  csshcmd = g_strdup (gtk_entry_get_text (GTK_ENTRY (cssh_cmd_entry)));
  gtk_widget_destroy (dialog);
  gtk_widget_show (gtk_window_list_toplevels ()->data);
  config_write ();

}

/* ###########################################################################*/
 
