/*
 * gtkpasman
 * Copyright (C) 2003 Cyril Bellot
 * Copyright (C) 2011 David Guglielmi
 *
 * passwd.c
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "passwd.h"

#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "gtkpasman_config.h"
#include "tree.h"

#define MAX_CMDLINE 1000

#define  EXLUDE_STR_VAL "gpg: WARNING:"

extern GList *data_list;

static int readpasswdline (char *, char **);
gchar *give_command (char *, char *, char *, char *);

int
filltree (GtkTreeStore *tree_store)
{

  char *buffer;
  char *res[8];

  int retval;
  int passpipe[2];
  int correct;
  int new_before_serviceplus;
  int i;

  FILE *fp;

  gchar *customer;
  gchar *groupe;
  gchar *server;
  gchar *service;
  gchar *serviceplus;

  gchar **data;

  GtkTreeIter iter_customer;
  GtkTreeIter iter_groupe;
  GtkTreeIter iter_host;
  GtkTreeIter iter_service;
  GtkTreeIter iter_serviceplus;
  GtkTreeIter iter_username;

  correct = 0;

  buffer = malloc (MAX_CMDLINE + 1);  
  bzero (buffer, MAX_CMDLINE + 1);

  retval = pipe (passpipe);
  retval = write (passpipe[1], globalpasswd, strlen (globalpasswd));
  close (passpipe[1]);

#ifdef DISABLE_GPG
  snprintf (buffer,\
            MAX_CMDLINE,\
	          "cat %s 2>&-",\
            passwdfile);
#else
  snprintf (buffer,\
            MAX_CMDLINE,\
	          "gpg --batch --no-tty --decrypt --passphrase-fd %d %s 2>&-",\
	          passpipe[0],\
            passwdfile);
#endif

  fp = popen (buffer, "r");

  if (fp == NULL)
   {
    fprintf (stderr, "Failed to open pipe\n");
    return 0;
   }

  customer = g_strdup ("");
  groupe = g_strdup ("");
  server = g_strdup ("");
  service = g_strdup ("");
  serviceplus = g_strdup ("");

  /* Main loop to read décrypted file */
  while (fgets (buffer, MAX_CMDLINE, fp) != NULL)
   {
     /* Exclude GnuPG warning lines */
    if (strncasecmp (buffer, EXLUDE_STR_VAL, sizeof (EXLUDE_STR_VAL -1)) == 0)
     continue;

    if (readpasswdline (buffer, res) == 7)
     {
		  correct = 1;
		  new_before_serviceplus = 0;

      if (strcmp (customer, res[1]))
       {
        append_tree_line (tree_store, res[1],\
          GTK_STOCK_ORIENTATION_PORTRAIT, &iter_customer, NULL);

        g_free (customer);
        customer = g_strdup (res[1]);
        g_free (groupe);
        groupe = g_strdup ("(none)");
        g_free (server);
        server = g_strdup ("(none)");
        g_free (service);
        service = g_strdup ("(none)");
        g_free (serviceplus);
        serviceplus = g_strdup ("(none)");
        new_before_serviceplus++;

       }

      if (strcmp (groupe, res[2]))
       {
	      /* new group */
	      if (strcmp (res[2], ""))
         {
		      /* new branch only is not empty */
          /* GTK_STOCK_YES */
            append_tree_line (tree_store, res[2], NULL, &iter_groupe,\
            &iter_customer);

         }
	        g_free (groupe);
	        groupe = g_strdup (res[2]);
	        g_free (server);
	        server = g_strdup ("(none)");
	        g_free (service);
	        service = g_strdup ("(none)");
	        g_free (serviceplus);
	        serviceplus = g_strdup ("(none)");
	        new_before_serviceplus++;
      }

     if (strcmp (server, res[3]))
      {
	       /* new server */
        if (strcmp (groupe, ""))
         {
          /* GTK_STOCK_HOME */
          append_tree_line (tree_store, res[3], NULL, &iter_host,\
            &iter_groupe);
        }
      else
       {
		    /* linked to customer if group is empty */
          /* GTK_STOCK_HOME */
          append_tree_line (tree_store, res[3], NULL, &iter_host,\
            &iter_customer);

        }

        g_free (server);
        server = g_strdup (res[3]);
        g_free (service);
        service = g_strdup ("(none)");
        g_free (serviceplus);
        serviceplus = g_strdup ("(none)");
        new_before_serviceplus++;
      }

      if (strcmp (service, res[4]))
       {
        /* new service */
        /* GTK_STOCK_CONNECT */
        append_tree_line (tree_store, res[4], NULL, &iter_service,\
          &iter_host);

        g_free (service);
        service = g_strdup (res[4]);
        g_free (serviceplus);
        serviceplus = g_strdup ("(none)");
        new_before_serviceplus++;

       }

		  if (strcmp (serviceplus, res[5]))
       {
	    /* new serviceplus */
	      if (strcmp (res[5], ""))
         {
		      /* new branch only if serviceplus is not empty */
          /* GTK_STOCK_CONNECT */
          append_tree_line (tree_store, res[5], NULL,\
            &iter_serviceplus, &iter_service);
	       }
	      g_free (serviceplus);
	      serviceplus = g_strdup (res[5]);
       }

      if (!strcmp (res[6], ""))
       {
        g_free (res[6]);
	      res[6] = g_strdup ("__none__");
		   }

		  /* new user */
		  if (strcmp (serviceplus, ""))
       {
	    /* linked to serviceplus if not empty */
        append_tree_line (tree_store, res[6], NULL, &iter_username,\
              &iter_serviceplus);
		   }
		  else
       {
	    /* otherwise linked to service */
        append_tree_line (tree_store, res[6], NULL, &iter_username,\
              &iter_service);
		   }

		  /* data (passwd, commands, infos, servername) */
      data = (gchar **) g_malloc (4 * sizeof (gchar *));
      data[0] = g_strdup (res[7]);
      data[1] = give_command ((char *) service, (char *) server,
				(char *) res[5], (char *) res[6]);	
      data[2] = g_strdup_printf ("%s %s %s", (char *) server,
				   (char *) service, (char *) res[6]);
      data[3] = g_strdup (server);

      gtk_tree_store_set (tree_store, &iter_username, TREE_COL_DATA, data, -1);

      data_list = g_list_append (data_list, data);

		  /* special case: ssh+root */
		  if ((!strcmp (res[6], "root")) && (!strcmp (res[4], "ssh")))
       {
	      data = (gchar **) g_malloc (4 * sizeof (gchar *));
	      data[0] = g_strdup (res[7]);
	      data[1] = give_command ((char *) service, (char *) server,
				      (char *) res[5], (char *) res[6]);	
	      data[2] = g_strdup_printf ("%s %s %s", (char *) server,
					     (char *) service, (char *) res[6]);
	      data[3] = g_strdup (server);

        gtk_tree_store_set (tree_store, &iter_host, TREE_COL_DATA, data, -1);

        data_list = g_list_append (data_list, data);

		   }

		  for (i = 1; i <= 7; i++)
       {
	      g_free (res[i]);
		   }

     }
    else
      fprintf (stderr, "WARNING: not matched: %s\n", buffer);

   }

  pclose (fp);
  free (buffer);

  return correct;

}

static int
readpasswdline (char *line, char *res[])
{
  gchar **fields;
  gchar tmp[512];
  gchar *tmp2;
  gint nb = 0, i;
  gchar *pattern[] = {
    "customer=%[^;]",
    "group=%[^;]",
    "server=%[^;]",
    "service=%[^;]",
    "serviceplus=%[^;]",
    "user=%[^;]",
    "password=%[^\n]"
  };

  /* Split fields */
  fields = g_strsplit (line, ";", 0);
  for (i = 0; i < 7; i++) {
    tmp[0] = tmp[1] = 0;

#ifdef __DEBUG
   /* FIXME: Uncomment here for debug */
   printf ("%s\n", fields[i]);
#endif
    /* strip spaces at start+end, and find pattern match */
    if (sscanf (g_strstrip (fields[i]), pattern[i], tmp)) {
      res[i + 1] = g_strdup (tmp);
      nb++;
    }
  }

  /* If passwd contains ; it will be cut. We stick them here */
  for (i = 7; fields[i] != NULL; i++) {
    tmp2 = g_strdup_printf ("%s;%s", res[7], fields[i]);
    g_free (res[7]);
    res[7] = g_strdup (tmp2);
    g_free (tmp2);
  }

  /* Free resources */
  for (i = 0; fields[i] != NULL; i++)
    g_free (fields[i]);

  /* Number of matched fields */
  return nb;

}

/*
 * emptytree_data
 * 	callback to free the 4 blocks of data linked to elements of the tree
 */
void
emptytree_data (gpointer data)
{

  g_free (((gchar **) data)[0]);
  g_free (((gchar **) data)[1]);
  g_free (((gchar **) data)[2]);
  g_free (((gchar **) data)[3]);
  g_free (((gchar **) data));

}

/*
 * give_command
 * 	give command to execute depending on service, replacing 
 * 	%u by username
 * 	%h by hostname
 */
gchar *
give_command (char *service, char *host, char *serviceplus, char *user)
{
  gchar c = ' ', *s;
  gchar *cmd, *res, *tmp;

  if (!g_strcasecmp (service, "Ssh")) {
    cmd = g_strdup (sshcmd);
  }
  else if (!g_strcasecmp (service, "Ftp")) {
    cmd = g_strdup (ftpcmd);
  }
  else if (!g_strcasecmp (service, "Http")) {
    cmd = g_strdup (httpcmd);
  }
  else if (!g_strcasecmp (service, "Telnet")) {
    cmd = g_strdup (telnetcmd);
  }
  else
    return g_strdup("");

  /* replace %u, %s or %h by values */
  res = g_strdup ("");
  tmp = g_strdup ("");
  for (s = cmd; *s != '\0'; ++s) {
    if (*s == '%' && *(s + 1) != '\0') {
      if ((c = *(s + 1)) == 'h')
		tmp = g_strconcat (res, host, NULL);
      else { if (c == 's')
				tmp = g_strconcat (res, serviceplus, NULL);
			 if (c == 'u')
					tmp = g_strconcat (res, user, NULL);
	  }
      s++;
    }
    else {
      tmp = g_strdup_printf ("%s%c", res, *s);
    }
    g_free (res);
    res = g_strdup (tmp);
    g_free (tmp);
  }

  /* free resources */
  g_free (cmd);

  return res;

}

