/*
 * gtkpasman
 * Copyright (C) 2003 Cyril Bellot
 * Copyrigth (c) 2011 David Guglielmi
 *
 * tree.c
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <string.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <gtk_ui/gtk_ui.h>

#include "tree.h"
#include "passwd.h"
#include "gtkpasman_config.h"

/* FIXME: Remove global variable */
GList *search_l;
GList *data_list;
int s_pos;

void free_search_l (GList *);
gboolean search_in_tree_view (GtkTreeModel *, GtkTreePath *, GtkTreeIter *,\
  gpointer);

GtkWidget*
pasman_tree_view_new ()
{

  GtkWidget *tree_view;
  GtkTreeStore *tree_store;
  GtkCellRenderer *cell;
  GtkTreeViewColumn *tree_column;

  tree_store = gtk_tree_store_new (N_COLUMNS,\
    GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_POINTER);

  tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));

  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (tree_view), FALSE);
  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (tree_view), FALSE);

  gtk_tree_selection_set_mode (\
    gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view)),\
    GTK_SELECTION_MULTIPLE);

  /* Column 1 */
  tree_column = gtk_tree_view_column_new ();
  cell = gtk_cell_renderer_pixbuf_new ();

  gtk_tree_view_column_pack_start (tree_column, cell, FALSE);
  gtk_tree_view_column_set_attributes (tree_column, cell, "pixbuf",\
    TREE_COL_PIXBUF,\
    NULL);

  gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), tree_column);

  /* Column 2 */
  tree_column = gtk_tree_view_column_new ();
  cell = gtk_cell_renderer_text_new ();

  gtk_tree_view_column_pack_start (tree_column, cell, FALSE);
  gtk_tree_view_column_set_attributes (tree_column, cell, "text",\
    TREE_COL_TEXT,\
    NULL);

  gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view), tree_column);
  gtk_tree_view_set_expander_column (GTK_TREE_VIEW (tree_view), tree_column);
  gtk_tree_view_column_set_max_width (tree_column, 80);

  g_signal_connect_swapped (tree_view, "destroy", G_CALLBACK (emptytree),\
                            tree_store);
  g_signal_connect_swapped (tree_view, "destroy", G_CALLBACK (free_search_l),\
                            search_l);

  return tree_view;
}

void
append_tree_line (GtkTreeStore *tree_store, gchar *text,\
  char *stock_id, GtkTreeIter *iter, GtkTreeIter *parent)
{

  GdkPixbuf *pixbuf;
  GtkWidget *toplevel;

  toplevel = gtk_window_list_toplevels ()->data;

  gtk_tree_store_append (tree_store, iter, parent);

  if (stock_id)
   {
#if GTK_CHECK_VERSION(3,0,0)
    pixbuf = gtk_widget_render_icon_pixbuf (toplevel,\
      stock_id,\
      GTK_ICON_SIZE_MENU);
#else
    pixbuf = gtk_widget_render_icon (toplevel,\
      stock_id,\
      GTK_ICON_SIZE_MENU,
      NULL);
#endif

    gtk_tree_store_set (tree_store,\
      iter,\
      TREE_COL_PIXBUF, pixbuf,\
      -1);

    g_object_unref (pixbuf);

  }

  gtk_tree_store_set (tree_store,\
    iter,\
    TREE_COL_TEXT, text,\
    -1);

}

void
button_copy_cmd_line (GtkTreeView *tree_view)
{

  GtkTreeSelection *selection;
  GtkTreeIter iter;
  GtkTreeModel *tree_model;
  GList *list;
  gchar **data;
 
  if (!decrypted_passwds)
    return;

  selection = gtk_tree_view_get_selection (tree_view);

  list = gtk_tree_selection_get_selected_rows (selection, &tree_model);
  gtk_tree_model_get_iter (tree_model, &iter, list->data);

  gtk_tree_model_get (tree_model, &iter, TREE_COL_DATA,  &data, -1);

  g_list_foreach (list, (GFunc) gtk_tree_path_free, NULL);
  g_list_free (list);

  select_command (data);

}

void
button_copy_passwd (GtkTreeView *tree_view)
{

  GtkTreeSelection *selection;
  GtkTreeIter iter;
  GtkTreeModel *tree_model;
  GList *list;
  gchar **data;

  if (!decrypted_passwds)
    return;

  selection = gtk_tree_view_get_selection (tree_view);

  list = gtk_tree_selection_get_selected_rows (selection, &tree_model);
  gtk_tree_model_get_iter (tree_model, &iter, list->data);

  gtk_tree_model_get (tree_model, &iter, TREE_COL_DATA,  &data, -1);

  g_list_foreach (list, (GFunc) gtk_tree_path_free, NULL);
  g_list_free (list);

  select_passwd (data);

}

void
button_copy_passwd_and_execute (GtkTreeView *tree_view)
{

  GtkTreeSelection *selection;
  GtkTreeIter iter;
  GtkTreeModel *tree_model;
  GList *list;
  gchar **data;
 
  if (!decrypted_passwds)
    return;

  selection = gtk_tree_view_get_selection (tree_view);

  list = gtk_tree_selection_get_selected_rows (selection, &tree_model);
  gtk_tree_model_get_iter (tree_model, &iter, list->data);

  gtk_tree_model_get (tree_model, &iter, TREE_COL_DATA,  &data, -1);

  g_list_foreach (list, (GFunc) gtk_tree_path_free, NULL);
  g_list_free (list);

  select_passwd_and_exec_command (data);

}

void
button_execute (GtkTreeView *tree_view)
{

  GtkTreeSelection *selection;
  GtkTreeIter iter;
  GtkTreeModel *tree_model;
  GList *list;
  GList *n;
  gchar **data;
  gchar *hostlist;
  int i;
 
  if (!decrypted_passwds)
    return;

  hostlist = g_strdup ("");

  selection = gtk_tree_view_get_selection (tree_view);

  list = gtk_tree_selection_get_selected_rows (selection, &tree_model);

  i = g_list_length (list);

  if (i > 1)
   {

	  for (n = list; n; n=n->next)
     {
      gtk_tree_model_get_iter (tree_model, &iter, n->data);
      gtk_tree_model_get (tree_model, &iter, TREE_COL_DATA,  &data, -1);

	    if (data)
        hostlist = g_strdup_printf("%s %s",hostlist, data[3]);
     }

    exec_multiple (hostlist);
   }
  else if (i == 1)
   {

      gtk_tree_model_get_iter (tree_model, &iter, list->data);
      gtk_tree_model_get (tree_model, &iter, TREE_COL_DATA,  &data, -1);

	    exec_command (data);
   }

	g_free (hostlist);

  g_list_foreach (list, (GFunc) gtk_tree_path_free, NULL);
  g_list_free (list);

}

void
search_in_list (GtkWidget *passtxt)
{
  GtkTreeModel *tree_model;
  GtkTreeView *tree_view;
  GtkWidget *window;

  free_search_l (search_l);

  window = gtk_window_list_toplevels ()->data;
  tree_view = g_object_get_data (G_OBJECT (window), "pasman_tree_view");
  tree_model = gtk_tree_view_get_model (tree_view);
    
  gtk_tree_model_foreach (tree_model,\
                          (GtkTreeModelForeachFunc)&search_in_tree_view,\
                          passtxt);
    
  
}

/* FIXME: Add search by host func
gboolean search_in_tree_view_by_host_only (GtkTreeModel *model,\
  GtkTreePath *path, GtkTreeIter *iter, gpointer data)
{

  gchar **s_data;

  gtk_tree_model_get (model, iter, TREE_COL_DATA,  &s_data, -1);

  if (s_data)
    printf ("%s\n", (char*) s_data[3]);

  return FALSE;

}
*/

gboolean search_in_tree_view (GtkTreeModel *model, GtkTreePath *path,\
  GtkTreeIter *iter, gpointer data)
{

  gchar *s_data;
  const gchar *pattern;
  GtkWidget *entry;
  GtkWidget *window;
  GtkTreeView *tree_view;
  gchar *a1;
  gchar *a2;
  int i; 
  char *str;
 
  entry = GTK_WIDGET (data);
  i = -1;

  gtk_tree_model_get (model, iter, TREE_COL_TEXT,  &s_data, -1);

  pattern = gtk_entry_get_text (GTK_ENTRY (entry));

  if (strlen (pattern) <= 2)
    return TRUE;

  a1 = g_ascii_strdown (s_data, -1);
  a2 = g_ascii_strdown (pattern, -1);

  str = strstr (a1, a2);
  
  if (str)
     i = strlen (str);

  g_free (a1);
  g_free (a2);

  /* string match */
  if (s_data && i >=  0)
   {

    search_l = g_list_append (search_l, gtk_tree_path_copy (path));

    if (g_list_length (search_l) ==  1)
     {
      window = gtk_window_list_toplevels ()->data;
      tree_view = g_object_get_data (G_OBJECT (window), "pasman_tree_view");
      gtk_tree_view_expand_to_path (tree_view, path);
      gtk_tree_view_set_cursor (tree_view, path, NULL,\
                                FALSE);
     }
#ifdef __DEBUG
    printf ("%s found.\n", (char*) s_data);
#endif
   }

  return FALSE;

}

void
find_next_in_tree_view (GtkButton *button, gpointer user_data)
{

  GtkTreePath *path;
  unsigned int max_pos;

  max_pos = g_list_length (search_l);
  s_pos = (max_pos - 1 > s_pos) ? s_pos + 1 : 0;

  if (max_pos <= 0)
    return;
  
  path = g_list_nth_data (search_l, s_pos);
  
  gtk_tree_view_expand_to_path (GTK_TREE_VIEW (user_data), path);
  gtk_tree_view_set_cursor (GTK_TREE_VIEW (user_data), path, NULL,\
                              FALSE);

}

void
find_prev_in_tree_view (GtkButton *button, gpointer user_data)
{

  GtkTreePath *path;
  unsigned int max_pos;
  
  max_pos = g_list_length (search_l);
  s_pos = (max_pos > s_pos - 1) ? s_pos - 1 : max_pos - 1;

  if (max_pos <= 0)
    return;

  path = g_list_nth_data (search_l, s_pos);
  
  gtk_tree_view_expand_to_path (GTK_TREE_VIEW (user_data), path);
  gtk_tree_view_set_cursor (GTK_TREE_VIEW (user_data), path, NULL,\
                              FALSE);

}

void
tree_expand_little (GtkTreeView *tree_view)
{

  GtkTreeModel *tree_model;
  GtkTreeIter iter;
  GtkTreeIter sub_iter;

  if (!decrypted_passwds)
    return;

  tree_model = gtk_tree_view_get_model (tree_view);

  if (!gtk_tree_model_get_iter_first (tree_model, &iter))
    return;

  do
   {
    gtk_tree_model_iter_children (tree_model, &sub_iter, &iter);

    do
     {
      gtk_tree_view_expand_to_path (tree_view,\
        gtk_tree_model_get_path (tree_model, &sub_iter));
     }
    while (gtk_tree_model_iter_next (tree_model, &sub_iter));

   }
  while (gtk_tree_model_iter_next (tree_model, &iter));

}

void
tree_unselectall (GtkTreeView *tree_view)
{

  gtk_tree_selection_unselect_all (\
    gtk_tree_view_get_selection (tree_view));

}

void
emptytree (GtkTreeStore *tree_store)
{

  gtk_tree_store_clear (tree_store);

  g_list_foreach (data_list, (GFunc)emptytree_data, NULL);
  g_list_free (data_list);

  data_list = NULL;

  decrypted_passwds = 0;

}

void free_search_l (GList *list)
{
   s_pos = 0;
   g_list_foreach (list, (GFunc)gtk_tree_path_free, NULL);
   g_list_free (list);
   search_l = NULL;
}
